@@ -32,21 +32,136 @@ except:
 class scrape(object):
         
     def newspaper_stories(words, search_type='or', search_level=0, urls=None, 
-                      display=True):
-        if urls == None:
-           news_urls = {
-                'huffington': 'http://huffingtonpost.com', 
-                'reuters':    'http://www.reuters.com', 
-                'cbs-news':   'http://www.cbsnews.com',
-                'usa-today':  'http://usatoday.com',
-                'cnn':        'http://cnn.com',
-                'npr':        'http://www.npr.org',
-                'abc-news':   'http://abcnews.com',
-                'us-news':    'http://www.usnews.com',
-                'msn':        'http://msn.com',
-                'pbs':        'http://www.pbs.org',
-                'nbc-news':   'http://www.nbcnews.com',
-                'fox':        'http://www.foxnews.com'}
+                      display=True, memorize=False, language='en'):
+        config = newspaper.Config()
+        config.memoize_articles = memorize
+        config.language         = language
+        config.fetch_images     = False
+        config.request_timeout  = 20
+        config.MIN_WORD_COUNT   = 300
+        config.MIN_SENT_COUNT   = 10
+        if urls == None or urls =='top_news':
+            news_urls = {
+            'huffington': 'http://huffingtonpost.com', 
+            'reuters':    'http://www.reuters.com', 
+            'cbs-news':   'http://www.cbsnews.com',
+            'usa-today':  'http://usatoday.com',
+            'cnn':        'http://cnn.com',
+            'npr':        'http://www.npr.org',
+            'abc-news':   'http://abcnews.com',
+            'us-news':    'http://www.usnews.com',
+            'msn':        'http://msn.com',
+            'pbs':        'http://www.pbs.org',
+            'nbc-news':   'http://www.nbcnews.com',
+            'msnbc':      'http://www.msnbc.com',
+            'fox':        'http://www.foxnews.com'}
+        elif urls=='all_us_news':
+            news_urls = {
+            'abc-news': 'https://abcnews.go.com',
+            'al-jazeera-english': 'http://www.aljazeera.com',
+            'ars-technica': 'http://arstechnica.com',
+            'associated-press': 'https://apnews.com/',
+            'axios': 'https://www.axios.com',
+            'bleacher-report': 'http://www.bleacherreport.com',
+            'bloomberg': 'http://www.bloomberg.com',
+            'breitbart-news': 'http://www.breitbart.com',
+            'business-insider': 'http://www.businessinsider.com',
+            'buzzfeed': 'https://www.buzzfeed.com',
+            'cbs-news': 'http://www.cbsnews.com',
+            'cnbc': 'http://www.cnbc.com',
+            'cnn': 'http://us.cnn.com',
+            'crypto-coins-news': 'https://www.ccn.com',
+            'engadget': 'https://www.engadget.com',
+            'entertainment-weekly': 'http://www.ew.com',
+            'espn': 'http://espn.go.com',
+            'espn-cric-info': 'http://www.espncricinfo.com/',
+            'fortune': 'http://fortune.com',
+            'fox-news': 'http://www.foxnews.com',
+            'fox-sports': 'http://www.foxsports.com',
+            'google-news': 'https://news.google.com',
+            'hacker-news': 'https://news.ycombinator.com',
+            'ign': 'http://www.ign.com',
+            'mashable': 'http://mashable.com',
+            'medical-news-today': 'http://www.medicalnewstoday.com',
+            'msnbc': 'http://www.msnbc.com',
+            'mtv-news': 'http://www.mtv.com/news',
+            'national-geographic': 'http://news.nationalgeographic.com',
+            'national-review': 'https://www.nationalreview.com/',
+            'nbc-news': 'http://www.nbcnews.com',
+            'new-scientist': 'https://www.newscientist.com/section/news',
+            'newsweek': 'http://www.newsweek.com',
+            'new-york-magazine': 'http://nymag.com',
+            'next-big-future': 'https://www.nextbigfuture.com',
+            'nfl-news': 'http://www.nfl.com/news',
+            'nhl-news': 'https://www.nhl.com/news',
+            'politico': 'https://www.politico.com',
+            'polygon': 'http://www.polygon.com',
+            'recode': 'http://www.recode.net',
+            'reddit-r-all': 'https://www.reddit.com/r/all',
+            'reuters': 'http://www.reuters.com',
+            'techcrunch': 'https://techcrunch.com',
+            'techradar': 'http://www.techradar.com',
+            'american-conservative': 'http://www.theamericanconservative.com/',
+            'hill': 'http://thehill.com',
+            'huffington-post': 'http://www.huffingtonpost.com',
+            'next-web': 'http://thenextweb.com',
+            'verge': 'http://www.theverge.com',
+            'wall-street-journal': 'http://www.wsj.com',
+            'washington-post': 'https://www.washingtonpost.com',
+            'washington-times': 'https://www.washingtontimes.com/',
+            'time': 'http://time.com',
+            'usa-today': 'http://www.usatoday.com/news',
+            'vice-news': 'https://news.vice.com',
+            'wired': 'https://www.wired.com'}
+        elif urls == "texas_universities":
+            news_urls = {
+            'A&M':               'http://www.tamu.edu',
+            'A&M-Commerce':      'http://www.tamuc.edu',
+            'A&M-Corpus':        'http://www.tamucc.edu',
+            'A&M-Kingsville':    'http://www.tamuk.edu',
+            'A&M-Galveston':     'http://www.tamug.edu',
+            'A&M-PrairieView':   'http://www.pvamu.edu',
+            'A&M-International': 'http://www.tamiu.edu',
+            'A&M-WestTexas':     'http://www.wtamu.edu',
+            'Baylor':            'http://www.baylor.edu',
+            'Rice':              'http://www.rice.edu',
+            'SFAustin':          'http://www.sfasu.edu',
+            'SMU':               'http://www.smu.edu',
+            'SulRoss':           'http://www.sulross.edu',
+            'TexasState':        'http://www.txstate.edu',
+            'Texas_Tech':        'http://www.ttu.edu',
+            'UDallas':           'http://www.udallas.edu',
+            'UHouston':          'http://www.uh.edu',
+            'UTexas':            'http://www.utexas.edu', 
+            'UT_Dallas':         'http://www.utdallas.edu',
+            'UT_ElPaso':         'http://www.utep.edu',
+            'UT_Houston':        'http://www.uth.edu',
+            'UT_NorthTexas':     'http://www.unt.edu',
+            'UT_SanAntonio':     'http://www.utsa.edu'}
+        elif urls == 'popular':
+            news_urls = {}
+            agency_urls = newspaper.popular_urls()
+            for i in range(len(agency_urls)):
+                val = agency_urls[i]
+                url = agency_urls[i].replace("http://", "")
+                url = url.replace("www.", "")
+                url = url.replace("blog.", "")
+                url = url.replace("blogs.", "")
+                url = url.replace(".com", "")
+                url = url.replace(".net", "")
+                url = url.replace(".au", "")
+                url = url.replace(".org", "")
+                url = url.replace(".co.uk", "")
+                url = url.replace("the", "")
+                url = url.replace(".", "-")
+                url = url.replace('usa', 'usa-')
+                if url=='berkeley-edu':
+                    continue
+                if url=='beta-na-leagueoflegends':
+                    continue
+                if url=='bottomline-as-ucsb-edu':
+                    continue
+                news_urls[url] = val
         else:
             news_urls = urls
         
@@ -71,12 +186,11 @@ class scrape(object):
                     
         df_articles = pd.DataFrame(columns=['agency', 'url', 'length', 
                                             'keywords', 'title', 'summary', 
-                                            'story'])
+                                            'text'])
         n_articles  = {}
         today       = str(date.today())
         for agency, url in news_urls.items():
-            paper   = newspaper.build(url, memoize_articles=False, \
-                                   fetch_images=False, request_timeout=20)
+            paper   = newspaper.build(url, config=config)
             if display:
                 print("\n{:>6d} Articles available from {:<s} on {:<10s}:".
                       format(paper.size(), agency.upper(), today))
@@ -130,9 +244,15 @@ class scrape(object):
                    url_lower.find('.se/' )     >=0 or \
                    url_lower.find('/se.' )     >=0 or \
                    url_lower.find('.es/' )     >=0 or \
-                   url_lower.find('/es.' )     >=0 :
+                   url_lower.find('/es.' )     >=0 or \
+                   url_lower.find('?button')   >=0 or \
+                   url_lower.find('calendar.') >=0 or \
+                   url_lower.find('calendar/') >=0 or \
+                   url_lower.find('/event/')   >=0 or \
+                   url_lower.find('engr.utexas') >=0 or \
+                   url_lower.find('sites.smu.')  >=0:
                        continue
-               
+
                 # Filter if search_level == 0, URL quick search
                 if search_level == 0:
                     # Verify url contains at least one of the key words
@@ -154,10 +274,15 @@ class scrape(object):
                 print("{:>6d} Selected for download".format(n_to_review))
             
             for article_url in article_collection:
-                article = Article(article_url)
-                article.download()
+                article = Article(article_url, config=config)
+                try:
+                    article.download()
+                except: 
+                    if display:
+                        print("Cannot download:", article_url[0:79])
+                    continue
                 n = 0
-                # Allow for a maximum of 5 download failures
+                # Limit download failures
                 stop_sec=1 # Initial max wait time in seconds
                 while n<2:
                     try:
@@ -168,7 +293,7 @@ class scrape(object):
                         # Initiate download again before new parse attempt
                         article.download()
                         # Timeout for 5 seconds waiting for download
-                        t0 = time()
+                        t0     = time()
                         tlapse = 0
                         while tlapse<stop_sec:
                             tlapse = time()-t0
@@ -183,13 +308,13 @@ class scrape(object):
                 keywords = article.keywords
                 title    = article.title
                 summary  = article.summary
-                story    = article.text
-                story_lower_case = story.lower()
+                text     = article.text
+                text_lower_case = text.lower()
                 if search_type == 'or':
                     found_it = False
                     # Verify the url contains at least one of the key words
                     for word in words:
-                        j = story_lower_case.find(word)
+                        j = text_lower_case.find(word)
                         if j>= 0:
                             found_it = True
                             break
@@ -197,18 +322,19 @@ class scrape(object):
                     # search type 'and'
                     found_it = True
                     for word in words:
-                        j = story_lower_case.find(word)
+                        j = text_lower_case.find(word)
                         if j < 0:
                             found_it = False
                             break
                 if found_it:
                     # Article contains words and passes filters
                     # Save this article for later full review
-                    length   = len(story)
-                    df_story = pd.DataFrame([[agency, url, length, keywords,
-                                              title, summary, story]], 
+                    length   = len(text)
+                    df_story = pd.DataFrame([[agency, article_url, length, 
+                                              keywords, title, summary, 
+                                              text]], 
                                 columns=['agency', 'url', 'length', 'keywords',
-                                             'title', 'summary', 'story'])
+                                             'title', 'summary', 'text'])
                     # Check for an identical already in the file
                     if df_articles.shape[0]==0:
                         df_articles  = df_articles.append(df_story)
@@ -216,7 +342,7 @@ class scrape(object):
                         # Verify this story is not already in df_articles
                         same_story = False
                         for i in range(df_articles.shape[0]):
-                            if story==df_articles['story'].iloc[i]:
+                            if text==df_articles['text'].iloc[i]:
                                 same_story   = True
                                 n_to_review -= 1
                                 continue
@@ -292,91 +418,60 @@ class scrape(object):
             m = len(pg)
         return pg.strip()
 
-    def newsapi_get_urls(search_words, key, urls=None):
-        if urls==None:
-            agency_urls = {
-            'abc-news': 'https://abcnews.go.com',
-            'al-jazeera-english': 'http://www.aljazeera.com',
-            'ars-technica': 'http://arstechnica.com',
-            'associated-press': 'https://apnews.com/',
-            'axios': 'https://www.axios.com',
-            'bleacher-report': 'http://www.bleacherreport.com',
-            'bloomberg': 'http://www.bloomberg.com',
-            'breitbart-news': 'http://www.breitbart.com',
-            'business-insider': 'http://www.businessinsider.com',
-            'buzzfeed': 'https://www.buzzfeed.com',
-            'cbs-news': 'http://www.cbsnews.com',
-            'cnbc': 'http://www.cnbc.com',
-            'cnn': 'http://us.cnn.com',
-            'crypto-coins-news': 'https://www.ccn.com',
-            'engadget': 'https://www.engadget.com',
-            'entertainment-weekly': 'http://www.ew.com',
-            'espn': 'http://espn.go.com',
-            'espn-cric-info': 'http://www.espncricinfo.com/',
-            'fortune': 'http://fortune.com',
-            'fox-news': 'http://www.foxnews.com',
-            'fox-sports': 'http://www.foxsports.com',
-            'google-news': 'https://news.google.com',
-            'hacker-news': 'https://news.ycombinator.com',
-            'ign': 'http://www.ign.com',
-            'mashable': 'http://mashable.com',
-            'medical-news-today': 'http://www.medicalnewstoday.com',
-            'msnbc': 'http://www.msnbc.com',
-            'mtv-news': 'http://www.mtv.com/news',
-            'national-geographic': 'http://news.nationalgeographic.com',
-            'national-review': 'https://www.nationalreview.com/',
-            'nbc-news': 'http://www.nbcnews.com',
-            'new-scientist': 'https://www.newscientist.com/section/news',
-            'newsweek': 'http://www.newsweek.com',
-            'new-york-magazine': 'http://nymag.com',
-            'next-big-future': 'https://www.nextbigfuture.com',
-            'nfl-news': 'http://www.nfl.com/news',
-            'nhl-news': 'https://www.nhl.com/news',
-            'politico': 'https://www.politico.com',
-            'polygon': 'http://www.polygon.com',
-            'recode': 'http://www.recode.net',
-            'reddit-r-all': 'https://www.reddit.com/r/all',
-            'reuters': 'http://www.reuters.com',
-            'techcrunch': 'https://techcrunch.com',
-            'techradar': 'http://www.techradar.com',
-            'the-american-conservative': 
-                        'http://www.theamericanconservative.com/',
-            'the-hill': 'http://thehill.com',
-            'the-huffington-post': 'http://www.huffingtonpost.com',
-            'the-new-york-times': 'http://www.nytimes.com',
-            'the-next-web': 'http://thenextweb.com',
-            'the-verge': 'http://www.theverge.com',
-            'the-wall-street-journal': 'http://www.wsj.com',
-            'the-washington-post': 'https://www.washingtonpost.com',
-            'the-washington-times': 'https://www.washingtontimes.com/',
-            'time': 'http://time.com',
-            'usa-today': 'http://www.usatoday.com/news',
-            'vice-news': 'https://news.vice.com',
-            'wired': 'https://www.wired.com'}
-        else:
-            agency_urls = urls
-        if len(search_words)==0 or agency_urls==None:
-            return None
-        print("Searching agencies for pages containing:", search_words)
-       
-        # Get your NEWSAPI key from https://newsapi.org/account
+    def newsapi_get_urls(apikey, search_words, urls=None):
         try:
-            api = NewsApiClient(api_key=key)
+            api = NewsApiClient(api_key=apikey)
         except:
-            raise RuntimeError("***Call to request_pages invalid.\n"+\
-                               " api key was not accepted.")
-            sys.exit()
-            
+            raise RuntimeError("APIKEY Invalid")
+        if len(search_words)==0 or search_words==None:
+            raise RuntimeError("No Search Words")
+        print("Searching agencies for pages containing:", search_words)
+        # This is my API key, each user must request their own
+        # API key from https://newsapi.org/account
+        api = NewsApiClient(api_key=apikey)
         api_urls  = []
+        # Note that newsapi only draws articles from registered sources
+        # These require a particular key/value combination in news_urls
+        # Even if the url is correct, if the key is not what is registered
+        # the search will be rejected for that agency
+        if urls == None or urls == 'top_news':
+               news_urls = { 
+                    'al-jazeera-english':      'http://www.aljazeera.com',
+                    'the-huffington-post':     'http://www.huffingtonpost.com',
+                    'bloomberg':               'http://www.bloomberg.com',
+                    'reuters':                 'http://www.reuters.com', 
+                    'cbs-news':                'http://www.cbsnews.com',
+                    'usa-today':               'http://www.usatoday.com/news',
+                    'cnn':                     'http://us.cnn.com',
+                    'abc-news':                'https://abcnews.go.com',
+                    'msnbc':                   'http://www.msnbc.com',
+                    'nbc-news':                'http://www.nbcnews.com',
+                    'the-wall-street-journal': 'http://www.wsj.com',
+                    'fox-news':                'http://www.foxnews.com',
+                    'associated-press':        'https://apnews.com/'}
+        elif urls=='all_us_news':
+                news_urls = {}
+                sources = api.get_sources()
+                n_sources = len(sources['sources'])
+                for i in range(n_sources):
+                    cay  = sources['sources'][i]['id']
+                    val  = sources['sources'][i]['url']
+                    lang = sources['sources'][i]['language']
+                    ctry = sources['sources'][i]['country']
+                    if lang == 'en' and ctry == 'us':
+                        news_urls[cay] = val
+        else:
+                news_urls = urls
         # Iterate over agencies and search words to pull more url's
-        # Limited to 1,000 requests/day - Likely to be exceeded 
-        for agency in agency_urls:
-            domain = agency_urls[agency].replace("http://", "")
-            print(agency, domain)
+        # Limited to 300 requests/day - Likely to be exceeded 
+        for agency in news_urls:
+            domain = news_urls[agency].replace("http://" , "")
+            domain = news_urls[agency].replace("https://", "")
+            print("{:.<30s} {:<50s}".format(agency, domain))
             for word in search_words:
                 # Get articles with q= in them, Limits to 20 URLs
                 try:
-                    articles = api.get_everything(q=word, language='en',\
+                    articles = api.get_everything(q=word, language='en',
                                         sources=agency, domains=domain)
                 except:
                     print("--->Unable to pull news from:", agency, "for", word)
@@ -391,47 +486,38 @@ class scrape(object):
         # Remove duplicates
         df_urls  = df_urls.drop_duplicates('url')
         n_unique = len(df_urls)
-        print("\nFound a total of", n_total, " URLs, of which", n_unique,\
+        print("\nFound a total of", n_total, " URLs, of which", n_unique,
               " were unique.")
         return df_urls
     
     def request_pages(df_urls):
-        try:
-            if df_urls.shape[0]==0:
-                return None
-        except:
-            raise RuntimeError("***Call to request_pages invalid.")
-            sys.exit()
-            
         web_pages = []
         for i in range(len(df_urls)):
             u   = df_urls.iloc[i]
             url = u[2]
-            k = len(url)
-            short_url = url[0:k]
-            short_url = short_url.replace("https://", "")
-            short_url = short_url.replace("http://", "")
-            k = len(short_url)
-            if k>70:
-                k=70
-            short_url = short_url[0:k]
+            short_url = url[0:50]
+            short_url = short_url.replace("https//", "")
+            short_url = short_url.replace("http//", "")
             n = 0
-            # Allow for a maximum of 3 download attempts
-            stop_sec=3 # Max wait time per attempt
+            # Allow for a maximum of 2 download failures
+            stop_sec=1 # Initial max wait time in seconds
             while n<2:
                 try:
                     r = requests.get(url, timeout=(stop_sec))
-                    if r.status_code == 408:
-                        print("-->HTML ERROR 408", short_url)
+                    if r.status_code == 404:
+                        print("-->HTML ERROR 404", short_url)
                         raise ValueError()
                     if r.status_code == 200:
-                        print(short_url)
+                        print("Obtained: "+short_url)
                     else:
                         print("-->Web page: "+short_url+" status code:", \
                                   r.status_code)
                     n=99
                     continue # Skip this page
                 except:
+                    if r.status_code == 404:
+                        n=99
+                        continue
                     n += 1
                     # Timeout waiting for download
                     t0 = time()
@@ -439,6 +525,8 @@ class scrape(object):
                     print("Waiting", stop_sec, "sec")
                     while tlapse<stop_sec:
                         tlapse = time()-t0
+                    # Double wait time if needed for next exception
+                    stop_sec = stop_sec*2
             if n != 99:
                 # download failed skip this page
                 continue
@@ -447,9 +535,12 @@ class scrape(object):
             page_text = scrape.clean_html(html_page)
             web_pages.append([url, page_text])
         df_www  = pd.DataFrame(web_pages, columns=['url', 'text'])
-        n_total  = len(df_www)
-        print("Attempted to download", len(df_urls), "web pages.", \
-              " Obtained", n_total, ".")
+        n_total  = len(df_urls)
+        # Remove duplicates
+        df_www  = df_www.drop_duplicates('url')
+        n_unique = len(df_urls)
+        print("Found a total of", n_total, " web pages, of which", n_unique,\
+              " were unique.")
         return df_www
     
 class Metrics:
